###### FileLock
{: id="20210415163536-h75snjp" updated="20210415163539"}

```
class FileLock {
public:
    FileLock() {}
    virtual ~FileLock() {}

    int fd_;
    std::string name_;

private:
    // No copying allowed
    FileLock(const FileLock&);
    void operator=(const FileLock&);
};
```
{: id="20210415163539-x2uopad" updated="20210415163540"}

#### Open
{: id="20210415134508-nfmzqn7" updated="20210415173213"}

##### `DoorPlate plate_;`的Init
{: id="20210415173245-a5ab7kc" fold="1" updated="20210415174226"}

对应的是dir_目录下的META文件. 对此有两个字段, dir_和fd_. 这个META文件会被解读为items数组(怀疑就是key的类型), 个数为32M
{: id="20210415173316-90od2dc" parent-fold="1" updated="20210415174049"}

```
private:
    std::string dir_;
    int fd_;
    Item* items_;
```
{: id="20210415134843-3tbqo5u" parent-fold="1" updated="20210415134845"}

##### `DataStore store_;`初始化
{: id="20210415173227-sizybet" parent-fold="1" updated="20210415173255"}

##### `FileLock* db_lock_;`初始化
{: id="20210415173209-lrj48dz" parent-fold="1" updated="20210415173227"}

{: id="20210415173257-bogmq4c" parent-fold="1"}

会打开文件(没有就创建), 创建会预留32M个items的大小
{: id="20210415173924-eo34ms7" updated="20210415174957"}

然后通过mmap创建到内存的映射, 如果是新创建的, memset清0, 通过`reinterpret_cast<Item*>`转换为Items数组, 存进items_字段, 这就是item_字段的作用, 代码见下
{: id="20210415174117-18abxzu" updated="20210415175258"}

##### DataStore
{: id="20210415134930-dod3he5" updated="20210415174449"}

```
int fd_;
std::string dir_;
Location next_location_;
```
{: id="20210415134933-49x6aiw" updated="20210415134953"}

```
// 对dir_/META, 如果没有就创建, 并且磁盘空间预留map_size那么大, fd_存入META文件的描述符, 无论有没有, items_都会映射为META文件, 大小为map_size, 这个数组的长度是kMaxDoorCnt, 也就是32M
RetCode DoorPlate::Init() {
    bool new_create = false; // 是看META文件是否存在的
    const int map_size = kMaxDoorCnt * sizeof(Item); //? 这不会就是存放key的空间大小吧?
		// 如果文件已经存在, map_size就完全用不到?
    if (!FileExists(dir_) && 0 != mkdir(dir_.c_str(), 0755)) {
        return kIOError;
    }

    std::string path = dir_ + "/" + kMetaFileName; // 这个文件的文件名就叫META
    int fd = open(path.c_str(), O_RDWR, 0644);
    if (fd < 0 && errno == ENOENT) {
        // not exist, then create
        fd = open(path.c_str(), O_RDWR | O_CREAT, 0644); // 可读可写
        if (fd >= 0) {
            new_create = true;
            if (posix_fallocate(fd, 0, map_size) != 0) { // 据说是保证给文件留这么大的空间, 使得write一定不会失败
                std::cerr << "posix_fallocate failed: " << strerror(errno)
                          << std::endl;
                close(fd);
                return kIOError;
            }
        }
    }
    if (fd < 0) {
        return kIOError;
    }
    fd_ = fd;

    void* ptr =
        mmap(NULL, map_size, PROT_READ | PROT_WRITE, MAP_SHARED, fd_, 0); // 把META文件映射到ptr数组中
    if (ptr == MAP_FAILED) {
        std::cerr << "MAP_FAILED: " << strerror(errno) << std::endl;
        close(fd);
        return kIOError;
    }
    if (new_create) {
        memset(ptr, 0, map_size); // 这么大的空间, 初始化为0, 相当于0初始化Items
    }

    items_ = reinterpret_cast<Item*>(ptr);
    return kSucc;
}
```
{: id="20210415174353-kmk08lu" fold="1" updated="20210415174353"}

### 语法
{: id="20210415134614-oo8q826" updated="20210415174243"}

> str
> Another string object, used entirely (or partially) as the comparing string.
> pos
> Position of the first character in the compared string.
> If this is greater than the string length, it throws out_of_range.
> Note: The first character is denoted by a value of 0 (not 1).
> len
> Length of compared string (if the string is shorter, as many characters as possible).
> A value of string::npos indicates all characters until the end of the string.
> {: id="20210415140324-cwgmkpx" fold="1"}
{: id="20210415140320-i5n91kh" updated="20210415140324"}

看起来,
{: id="20210415140330-yrcg939" updated="20210415140335"}

###### posix_fallocate
{: id="20210415144538-leh1nys" fold="1" updated="20210415145431"}

posix_fallocate(fd, offset, len)
offset总不会是对disk起作用的吧?
that ensures that space is allocated on disk for the byte range specified by offset and len for the disk file referred to by the descriptor fd.
This allows an application to be sure that a later write() to the file won’t fail because disk space is exhausted (which could otherwise occur if a hole in the file was filled in, or some other application con- sumed space on the disk).
{: id="20210415145420-qlv3hfc" parent-fold="1" updated="20210415145424"}

```
struct flock {
    short l_type; /* Lock type: F_RDLCK, F_WRLCK, F_UNLCK */
    short l_whence; /* How to interpret 'l_start': SEEK_SET, SEEK_CUR, SEEK_END */

off_t l_start; /* Offset where the lock begins */
off_t l_len;  /* Number of bytes to lock; 0 means "until EOF" */
pid_t l_pid; /* Process preventing our lock (F_GETLK only) */
};
```
{: id="20210415152321-ma0y11v" parent-fold="1" updated="20210415152427"}

{: id="20210415152414-3f0svtb" parent-fold="1"}


{: id="20210415134433-h1ixi61" type="doc"}
